@@ -271,11 +271,6 @@ int main(int argc, char **argv)
     stdout_watch->activity.connect(sigc::ptr_fun(&stdout_handler));
 
       /* Redirect stdout to the logpipe */
-    if (close(STDOUT_FILENO) == -1)
-    {
-      perror("close(stdout)");
-      exit(1);
-    }
     if (dup2(pipefd[1], STDOUT_FILENO) == -1)
     {
       perror("dup2(stdout)");
@@ -283,19 +278,28 @@ int main(int argc, char **argv)
     }
 
       /* Redirect stderr to the logpipe */
-    if (close(STDERR_FILENO) == -1)
-    {
-      perror("close(stderr)");
-      exit(1);
-    }
     if (dup2(pipefd[1], STDERR_FILENO) == -1)
     {
       perror("dup2(stderr)");
       exit(1);
     }
 
-      /* Close stdin */
-    close(STDIN_FILENO);
+      // We also need to close stdin but that is not a good idea since we need
+      // the stdin filedescriptor to keep being allocated so that it is not
+      // assigned to some other random filedescriptor allocation. That would
+      // be very bad.
+    int devnull = open("/dev/null", O_RDONLY);
+    if (devnull == -1)
+    {
+      perror("open(/dev/null)");
+      exit(1);
+    }
+    if (dup2(devnull, STDIN_FILENO) == -1)
+    {
+      perror("dup2(stdin)");
+      exit(1);
+    }
+    close(devnull);
     
       /* Force stdout to line buffered mode */
     if (setvbuf(stdout, NULL, _IOLBF, 0) != 0)
